==> react-image-annotate-1.2.1.tgz: package/ImageMask/index.js <==
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useState, useEffect, useMemo, useRef } from "react";
import { colorInts } from "../colors";
import { useDebounce } from "react-use";
import MMGC_INIT from "mmgc1-cpp";
export var ImageMask = function ImageMask(_ref) {
  var regions = _ref.regions,
      regionClsList = _ref.regionClsList,
      imageSrc = _ref.imageSrc,
      imagePosition = _ref.imagePosition,
      _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? 1 : _ref$zIndex,
      _ref$hide = _ref.hide,
      hide = _ref$hide === void 0 ? false : _ref$hide;
  if (!window.mmgc) window.mmgc = MMGC_INIT();
  var mmgc = window.mmgc;

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      canvasRef = _useState2[0],
      setCanvasRef = _useState2[1];

  var superPixelsGenerated = useRef(false);

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      sampleImageData = _useState4[0],
      setSampleImageData = _useState4[1];

  useEffect(function () {
    if (!imageSrc) return;
    var canvas = document.createElement("canvas");
    var ctx = canvas.getContext("2d");
    var image = new Image();
    image.crossOrigin = "anonymous";
    image.src = imageSrc;

    image.onload = function () {
      canvas.width = image.naturalWidth;
      canvas.height = image.naturalHeight;
      ctx.drawImage(image, 0, 0);
      var imageData = ctx.getImageData(0, 0, image.naturalWidth, image.naturalHeight);
      superPixelsGenerated.current = false;
      setSampleImageData(imageData);
    };
  }, [imageSrc]);
  useDebounce(function () {
    if (hide) return;
    if (!canvasRef) return;
    if (!sampleImageData) return;
    if (regions.filter(function (cp) {
      return cp.cls;
    }).length < 3) return;
    if (!mmgc.setImageSize) return;
    var context = canvasRef.getContext("2d");

    if (!superPixelsGenerated.current) {
      superPixelsGenerated.current = "processing";
      mmgc.setMaxClusters(10000);
      mmgc.setImageSize(sampleImageData.width, sampleImageData.height);
      mmgc.setClassColor(0, 0);

      for (var i = 0; i < colorInts.length; i++) {
        mmgc.setClassColor(i + 1, colorInts[i]);
      }

      var imageAddress = mmgc.getImageAddr();
      mmgc.HEAPU8.set(sampleImageData.data, imageAddress);
      mmgc.computeSuperPixels();
      superPixelsGenerated.current = "done";
    }

    if (superPixelsGenerated.current !== "done") return; // mmgc.setVerboseMode(true)

    if (!["bg", "background", "nothing"].includes(regionClsList[0].toLowerCase())) {
      console.log("first region cls must be \"bg\" or \"background\" or \"nothing\"");
      return;
    }

    mmgc.clearClassElements();
    var classPoints = regions.filter(function (r) {
      return r.type === "point";
    }).filter(function (r) {
      return r.cls;
    });
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = classPoints[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var classPoint = _step.value;
        if (classPoint.x < 0 || classPoint.x >= 1) continue;
        if (classPoint.y < 0 || classPoint.y >= 1) continue;
        var clsIndex = regionClsList.indexOf(classPoint.cls);

        if (clsIndex > colorInts.length) {
          console.log("Too many classes to draw on mask!");
          continue;
        }

        mmgc.addClassPoint(clsIndex, Math.floor(classPoint.y * sampleImageData.height), Math.floor(classPoint.x * sampleImageData.width));
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    var classPolygons = regions.filter(function (r) {
      return r.type === "polygon";
    }).filter(function (r) {
      return r.cls;
    });
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      var _loop = function _loop() {
        var polygon = _step2.value;
        var points = polygon.points;
        var clsIndex = regionClsList.indexOf(polygon.cls);
        var pi = mmgc.addPolygon(clsIndex);
        var pointPairs = points.map(function (p, i) {
          return [p, points[(i + 1) % points.length]];
        });
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = pointPairs[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _ref4 = _step3.value;

            var _ref3 = _slicedToArray(_ref4, 2);

            var p1 = _ref3[0];
            var p2 = _ref3[1];
            var ri1 = Math.round(p1[1] * sampleImageData.height);
            var ci1 = Math.round(p1[0] * sampleImageData.width);
            var ri2 = Math.round(p2[1] * sampleImageData.height);
            var ci2 = Math.round(p2[0] * sampleImageData.width);
            mmgc.addLineToPolygon(pi, ri1, ci1, ri2, ci2);
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      };

      for (var _iterator2 = classPolygons[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        _loop();
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    mmgc.computeMasks();
    var maskAddress = mmgc.getColoredMask();
    var cppImDataUint8 = new Uint8ClampedArray(mmgc.HEAPU8.buffer, maskAddress, sampleImageData.data.length);
    var maskImageData = new ImageData(cppImDataUint8, sampleImageData.width, sampleImageData.height);
    context.clearRect(0, 0, sampleImageData.width, sampleImageData.height);
    context.putImageData(maskImageData, 0, 0);
  }, 1000, [canvasRef, sampleImageData, regions, hide]);
  var style = useMemo(function () {
    var width = imagePosition.bottomRight.x - imagePosition.topLeft.x;
    var height = imagePosition.bottomRight.y - imagePosition.topLeft.y;
    return {
      display: hide ? "none" : undefined,
      imageRendering: "pixelated",
      transform: "translateZ(0px)",
      left: imagePosition.topLeft.x,
      top: imagePosition.topLeft.y,
      width: isNaN(width) ? 0 : width,
      height: isNaN(height) ? 0 : height,
      zIndex: zIndex,
      position: "absolute",
      pointerEvents: "none"
    };
  }, [imagePosition.topLeft.x, imagePosition.topLeft.y, imagePosition.bottomRight.x, imagePosition.bottomRight.y, zIndex, hide]);
  return React.createElement("canvas", {
    style: style,
    width: sampleImageData ? sampleImageData.width : 0,
    height: sampleImageData ? sampleImageData.height : 0,
    ref: setCanvasRef
  });
};
export default ImageMask;

==> react-image-annotate-1.2.4.tgz: package/ImageMask/index.js <==
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useState, useEffect, useMemo, useRef } from "react";
import { colorInts } from "../colors";
import { useDebounce } from "react-use";
import MMGC_INIT from "mmgc1-cpp";
export var ImageMask = function ImageMask(_ref) {
  var regions = _ref.regions,
      regionClsList = _ref.regionClsList,
      imageSrc = _ref.imageSrc,
      imagePosition = _ref.imagePosition,
      _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? 1 : _ref$zIndex,
      _ref$hide = _ref.hide,
      hide = _ref$hide === void 0 ? false : _ref$hide,
      _ref$autoSegmentation = _ref.autoSegmentationOptions,
      autoSegmentationOptions = _ref$autoSegmentation === void 0 ? {} : _ref$autoSegmentation;
  if (!window.mmgc) window.mmgc = MMGC_INIT();
  var mmgc = window.mmgc;

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      canvasRef = _useState2[0],
      setCanvasRef = _useState2[1];

  var superPixelsGenerated = useRef(false);

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      sampleImageData = _useState4[0],
      setSampleImageData = _useState4[1];

  useEffect(function () {
    if (!imageSrc) return;
    var canvas = document.createElement("canvas");
    var ctx = canvas.getContext("2d");
    var image = new Image();
    image.crossOrigin = "anonymous";
    image.src = imageSrc;

    image.onload = function () {
      canvas.width = image.naturalWidth;
      canvas.height = image.naturalHeight;
      ctx.drawImage(image, 0, 0);
      var imageData = ctx.getImageData(0, 0, image.naturalWidth, image.naturalHeight);
      superPixelsGenerated.current = false;
      setSampleImageData(imageData);
    };
  }, [imageSrc]);
  useDebounce(function () {
    if (hide) return;
    if (!canvasRef) return;
    if (!sampleImageData) return;
    if (regions.filter(function (cp) {
      return cp.cls;
    }).length < 3) return;
    if (!mmgc.setImageSize) return;
    var context = canvasRef.getContext("2d");

    if (!superPixelsGenerated.current) {
      superPixelsGenerated.current = "processing";
      mmgc.setSimpleMode(autoSegmentationOptions.type === "simple");
      mmgc.setMaxClusters(10000);
      mmgc.setImageSize(sampleImageData.width, sampleImageData.height);
      mmgc.setClassColor(0, 0);

      for (var i = 0; i < colorInts.length; i++) {
        mmgc.setClassColor(i + 1, colorInts[i]);
      }

      var imageAddress = mmgc.getImageAddr();
      mmgc.HEAPU8.set(sampleImageData.data, imageAddress);
      mmgc.computeSuperPixels();
      superPixelsGenerated.current = "done";
    }

    if (superPixelsGenerated.current !== "done") return; // mmgc.setVerboseMode(true)

    if (!["bg", "background", "nothing"].includes(regionClsList[0].toLowerCase())) {
      console.log("first region cls must be \"bg\" or \"background\" or \"nothing\"");
      return;
    }

    mmgc.clearClassElements();
    var classPoints = regions.filter(function (r) {
      return r.type === "point";
    }).filter(function (r) {
      return r.cls;
    });
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = classPoints[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var classPoint = _step.value;
        if (classPoint.x < 0 || classPoint.x >= 1) continue;
        if (classPoint.y < 0 || classPoint.y >= 1) continue;
        var clsIndex = regionClsList.indexOf(classPoint.cls);

        if (clsIndex > colorInts.length) {
          console.log("Too many classes to draw on mask!");
          continue;
        }

        mmgc.addClassPoint(clsIndex, Math.floor(classPoint.y * sampleImageData.height), Math.floor(classPoint.x * sampleImageData.width));
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    var classPolygons = regions.filter(function (r) {
      return r.type === "polygon";
    }).filter(function (r) {
      return r.cls;
    });
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      var _loop = function _loop() {
        var polygon = _step2.value;
        var points = polygon.points;
        var clsIndex = regionClsList.indexOf(polygon.cls);
        var pi = mmgc.addPolygon(clsIndex);
        var pointPairs = points.map(function (p, i) {
          return [p, points[(i + 1) % points.length]];
        });
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = pointPairs[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _ref4 = _step3.value;

            var _ref3 = _slicedToArray(_ref4, 2);

            var p1 = _ref3[0];
            var p2 = _ref3[1];
            var ri1 = Math.round(p1[1] * sampleImageData.height);
            var ci1 = Math.round(p1[0] * sampleImageData.width);
            var ri2 = Math.round(p2[1] * sampleImageData.height);
            var ci2 = Math.round(p2[0] * sampleImageData.width);
            mmgc.addLineToPolygon(pi, ri1, ci1, ri2, ci2);
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      };

      for (var _iterator2 = classPolygons[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        _loop();
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    mmgc.computeMasks();
    var maskAddress = mmgc.getColoredMask();
    var cppImDataUint8 = new Uint8ClampedArray(mmgc.HEAPU8.buffer, maskAddress, sampleImageData.data.length);
    var maskImageData = new ImageData(cppImDataUint8, sampleImageData.width, sampleImageData.height);
    context.clearRect(0, 0, sampleImageData.width, sampleImageData.height);
    context.putImageData(maskImageData, 0, 0);
  }, 1000, [canvasRef, sampleImageData, regions, hide]);
  var style = useMemo(function () {
    var width = imagePosition.bottomRight.x - imagePosition.topLeft.x;
    var height = imagePosition.bottomRight.y - imagePosition.topLeft.y;
    return {
      display: hide ? "none" : undefined,
      imageRendering: "pixelated",
      transform: "translateZ(0px)",
      left: imagePosition.topLeft.x,
      top: imagePosition.topLeft.y,
      width: isNaN(width) ? 0 : width,
      height: isNaN(height) ? 0 : height,
      zIndex: zIndex,
      position: "absolute",
      pointerEvents: "none"
    };
  }, [imagePosition.topLeft.x, imagePosition.topLeft.y, imagePosition.bottomRight.x, imagePosition.bottomRight.y, zIndex, hide]);
  return React.createElement("canvas", {
    style: style,
    width: sampleImageData ? sampleImageData.width : 0,
    height: sampleImageData ? sampleImageData.height : 0,
    ref: setCanvasRef
  });
};
export default ImageMask;
